<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Controller;

use sgae3\Helper\Sacct;
use sgae3\Service\QueueService;
use sgae3\Helper\AjaxMessages;

use Exception;

class CronController
{

    public function runCron()
    {

        //Abort if qacct is already running.
        if ($this->checkCronStatus()) {
            die("Cron already executing. Aborting\n");
        }
        $pid = getmypid();
        if (!$this->cronLock($pid)) {
            die("Data parsing aborted. See error log for details");
        }

        echo "[".date("D d M H:i:s Y")."] "."Cron started" . PHP_EOL;

        $sacct = new sacct();

        $queue_service = new QueueService();
        $queues = $queue_service->getEnabledQueues();
        $min_year = $GLOBALS['min_year'];

        foreach ($queues as $queue) {

            $queue_name = $queue->getQueueName();
            echo "[".date("D d M H:i:s Y")."] "."Parsing $queue_name" . PHP_EOL;
            $lastCheckedDate = $queue->getLastCheckedDate();

            // BUG: sacct flag -S does not work with "1970-01-01T01:00:00"
            // HACK: use year 2000+ instead
            if ($lastCheckedDate == 0) {
                $min_year = $GLOBALS['min_year'] < 2000 ? 2000 : $GLOBALS['min_year'];
                $from = "$min_year-01-01T00:00:00";
            } else {
                //Add +1 second to avoid overlapping and repeated ID's
                $from = date("Y-m-d\TH:i:s", $queue->getLastCheckedDate() + 1);
            }
            $to = date("Y-m-d\TH:i:s");

            try {
                $sacct->import($queue_name, $from, $to);
            } catch (Exception $e) {
                AjaxMessages::ajaxError($e->getMessage());
            }

            try {
                $queue->setLastCheckedDate(strtotime($to));
                $queue_service->update($queue);
            } catch (Exception $e) {
                AjaxMessages::ajaxError("Unable to update Last Checked Date '$to' for '$queue_name'");
            }
        }
        $this->cronUnlock();
        echo "[".date("D d M H:i:s Y")."] "."Cron finished" . PHP_EOL;
    }

    public function checkCronStatus()
    {
        $sgae_lck_file = $GLOBALS['lock_file'];
        if (file_exists($sgae_lck_file)) {
            if (!$fd = @fopen($sgae_lck_file, "r")) {
                trigger_error("Can't open $sgae_lck_file", E_USER_ERROR);
                return false;
            }
            if (!$pid = @fread($fd, filesize($sgae_lck_file))) {
                trigger_error("Can't read $sgae_lck_file", E_USER_ERROR);
                return false;
            }
            $running = 0;
            $running = trim(exec("ps -e | grep $pid | wc -l"));
            if ($running == 0) {
                trigger_error("Cron process has died! cleaning lock file...\n");
                unlink($sgae_lck_file);
                return false;
            }
            return true;
        }
        return false;
    }

    private function cronLock($pid)
    {
        $sgae_lck_file = $GLOBALS['lock_file'];
        if (!$fd = @fopen($sgae_lck_file, "w")) {
            trigger_error("Can't create $sgae_lck_file", E_USER_ERROR);
            return false;
        }
        if (!@fwrite($fd, "$pid")) {
            trigger_error("Can't write on $sgae_lck_file", E_USER_ERROR);
            return false;
        }
        fclose($fd);
        return true;
    }

    private function cronUnlock()
    {
        $sgae_lck_file = $GLOBALS['lock_file'];
        unlink($sgae_lck_file);
    }
}
