<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Controller;

use Exception;
use sgae3\Service\GraphsService;
use sgae3\Helper\AjaxMessages;
use sgae3\Service\QueueService;
use sgae3\Helper\Squeue;

class GraphsController
{

    // From user view
    public function userGraphs()
    {
        $queue = $this->parseQueue();
        return $this->userOrUserQueueGraphs($_SESSION['username'], $queue);
    }

    private function loggedUserIsQueueAdmin(string $queue)
    {
        $qs = new QueueService();
        if ($_SESSION['role'] != "admin") {
            if ($_SESSION['role'] == "queue_admin") {
                if (!$qs->userIsQueueAdmin($_SESSION['username'], $queue)) {
                    AjaxMessages::ajaxError("You are not an administrator of '$queue'");
                }
            } else {
                AjaxMessages::ajaxError("Forbidden");
            }
        }
    }

    // From queue view providing username
    public function userQueueGraphs()
    {
        $username = $this->parseUsername();
        $queue = $this->parseQueue();
        $this->loggedUserIsQueueAdmin($queue);
        return $this->userOrUserQueueGraphs($username, $queue);
    }

    // Both user view and queue view providing username
    public function userOrUserQueueGraphs($username, $queue)
    {
        list($from, $to) = $this->parseDate();

        $gs = new GraphsService();
        $sq = new Squeue();

        try {
            AjaxMessages::ajaxData(array_merge(
                $sq->user_live_data($username, $queue),
                $gs->userGraphs($username, $queue, $from, $to)
            ));
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
    }

    // Queue view without providing username
    public function queueGraphs()
    {
        list($from, $to) = $this->parseDate();
        $queue = $this->parseQueue();
        $this->loggedUserIsQueueAdmin($queue);

        $gs = new GraphsService();
        $sq = new Squeue();

        try {
            AjaxMessages::ajaxData(array_merge(
                $sq->queue_live_data($queue),
                $gs->queueGraphs($queue, $from, $to)
            ));
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
    }

    // From cluster view providing username
    public function userClusterGraphs()
    {
        if ($_SESSION['role'] != "admin") {
            AjaxMessages::ajaxError("Forbidden");
        }

        if (!isset($_POST['username']) || $_POST['username'] == "") {
            AjaxMessages::ajaxError("Invalid username");
        }
        list($from, $to) = $this->parseDate();

        $gs = new GraphsService();
        $sq = new Squeue();

        try {
            AjaxMessages::ajaxData(array_merge(
                $sq->user_cluster_live_data($_POST['username']),
                $gs->userClusterGraphs($_POST['username'], $from, $to)
            ));
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
    }

    // From cluster view without providing username
    public function clusterGraphs()
    {
        if ($_SESSION['role'] != "admin") {
            AjaxMessages::ajaxError("Forbidden");
        }

        list($from, $to) = $this->parseDate();

        $gs = new GraphsService();
        $sq = new Squeue();

        try {
            AjaxMessages::ajaxData(array_merge(
                $sq->cluster_live_data(),
                $gs->clusterGraphs($from, $to)
            ));
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
    }

    // From yearly view
    public function yearlyGraphs()
    {
        if ($_SESSION['role'] != "admin") {
            AjaxMessages::ajaxError("Forbidden");
        }
        $gs = new GraphsService();
        try {
            AjaxMessages::ajaxData($gs->yearlyGraphs());
        } catch (Exception $e) {
            AjaxMessages::ajaxError($e->getMessage());
        }
    }

    // Parse queue name from POST
    private function parseQueue()
    {

        if (!isset($_POST['queue_select']) || $_POST['queue_select'] == "") {
            AjaxMessages::ajaxError("Invalid Queue");
        }

        return $_POST['queue_select'];
    }

    // Parse queue name from POST
    private function parseUsername()
    {

        if (!isset($_POST['username']) || $_POST['username'] == "") {
            AjaxMessages::ajaxError("Invalid Username");
        }

        return $_POST['username'];
    }

    // Parse date from POST
    private function parseDate()
    {
        $datetype = $_POST['datetype'];
        $year_select = $_POST['year_select'];

        if ($datetype == "full_year") {
            $from = strtotime("$year_select-01-01T00:00");
            $to = strtotime("$year_select-12-31T23:59");
        } else if ($datetype == "date_range") {

            if (!strtotime($_POST['start_time'])) {
                AjaxMessages::ajaxError("Invalid Start date");
            }
            if (!strtotime($_POST['end_time'])) {
                AjaxMessages::ajaxError("Invalid Start date");
            }

            list($start_year, $start_month, $start_day) = preg_split("/\-/", $_POST['start_time']);
            $from = mktime(0, 0, 0, $start_month, $start_day, $start_year);
            list($end_year, $end_month, $end_day) = preg_split("/\-/", $_POST['end_time']);
            $to = mktime(23, 59, 59, $end_month, $end_day, $end_year);
        } else {
            AjaxMessages::ajaxError("Unknown datetype");
        }

        if ($from > $to) {
            AjaxMessages::ajaxError("End date is before Start date");
        }

        return array($from, $to);
    }
}
