<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Controller;

use Exception;
use sgae3\Service\AdminService;
use sgae3\Service\DBLoginService;
use sgae3\Service\HtaccessLoginService;
use sgae3\Service\QueueAdminService;
use sgae3\Helper\AjaxMessages;

class LoginController
{

    public function checkLogged()
    {
        $rC = new RouteController();
        list($request_uri, $request_method) = $rC->parseRequest();

        //Break infinite redirect loop
        if ($request_uri == "/login") {
            return;
        }

        if (!isset($_SESSION['username'])) {

            $loginBackend = $GLOBALS['login_backend'];
            switch ($loginBackend) {

                case "htaccess":
                    $loginService = new HtaccessLoginService();
                    if (!$loginService->islogged("")) {
                        http_response_code(401);
                        require $GLOBALS['app_path'] . '/views/401.phtml';
                        exit();
                    } else {
                        $_SESSION['REQUEST_URI'] = $request_uri;
                        $loginService->doLogin("", "");
                        $this->redirect();
                    }
                    break;

                case "localDB":
                    if ($request_method == "GET") {
                        // Force login
                        $_SESSION['REQUEST_URI'] = $request_uri;
                        http_response_code(302);
                        header('Location: ' . $GLOBALS['app_url'] . '/login');
                        exit();
                    } else {
                        http_response_code(401);
                        AjaxMessages::ajaxError("Login required");
                        exit();
                    }
                    break;

                default:
                    http_response_code(401);
                    require $GLOBALS['app_path'] . '/views/401.phtml';
                    exit();
            }
        }

        $this->setRole();
    }

    /**
     * Save user role at SESSION
     */
    public function setRole()
    {
        $username = $_SESSION['username'];
        $adminService = new AdminService();
        $queueAdminService = new QueueAdminService();

        if ($adminService->isAdmin($username)) {
            $_SESSION['role'] = "admin";
        } else if ($queueAdminService->isQueueAdmin($username)) {
            $_SESSION['role'] = "queue_admin";
        } else {
            $_SESSION['role'] = "user";
        }
    }

    /**
     * Go to / or REQUEST URI *before login* (if exists)
     */
    public function redirect()
    {
        $uri = "";
        if (isset($_SESSION['REQUEST_URI'])) {
            $uri = $_SESSION['REQUEST_URI'];
            unset($_SESSION['REQUEST_URI']);
        }
        http_response_code(302);
        header('Location: ' . $GLOBALS['app_url'] . $uri);
    }

    /**
     * Admin routes protection 
     */
    public function checkIsAdmin()
    {
        $request_method = $_SERVER['REQUEST_METHOD'];

        if ($_SESSION['role'] != "admin") {
            http_response_code(403);
            if ($request_method == "GET") {
                require $GLOBALS['app_path'] . '/views/403.phtml';
                exit();
            } else {
                AjaxMessages::ajaxError("Forbidden");
            }
        }
    }

    /**
     * Show login form for localDB login
     */
    public function loginPage()
    {
        if (isset($_SESSION['username'])) {
            http_response_code(302);
            header('Location: ' . $GLOBALS['app_url']);
        }
        require $GLOBALS['app_path'] . '/views/login.phtml';
    }

    /**
     * Process login form for localDB login
     */
    public function loginPost()
    {
        $loginService = new DBLoginService();

        if ($_POST['username'] == "") {
            AjaxMessages::ajaxError("Please provide a user");
        }
        $username = $_POST['username'];

        if ($_POST['password'] == "") {
            AjaxMessages::ajaxError("Please provide a password");
        }
        $password = $_POST['password'];

        try {
            if ($loginService->doLogin($username, $password)) {
                $uri = "";
                if (isset($_SESSION['REQUEST_URI'])) {
                    $uri = $_SESSION['REQUEST_URI'];
                    unset($_SESSION['REQUEST_URI']);
                }
                AjaxMessages::ajaxData($GLOBALS['app_url'] . $uri);
            } else {
                AjaxMessages::ajaxError("Invalid credentials");
            }
        } catch (Exception $e) {
            AjaxMessages::ajaxError("Service temporary unavailable.");
        }
    }

    /**
     * Process logout
     */
    public function logout()
    {
        // Destruir todas las variables de sesión.
        $_SESSION = array();

        // Si se desea destruir la sesión completamente, borre también la cookie de sesión.
        if (ini_get("session.use_cookies")) {
            $params = session_get_cookie_params();
            setcookie(session_name(), '', time() - 42000, $params["path"], $params["domain"], $params["secure"], $params["httponly"]);
        }

        // Finalmente, destruir la sesión.
        session_destroy();

        // Redirect to index.php page
        header('Location: ' . $GLOBALS['app_url']);
        die();
    }
}
